/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package be.ac.vub.cocompose.eclipse.model.core;

import java.util.List;
import java.util.Vector;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.ui.views.properties.ColorPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

import be.ac.vub.cocompose.eclipse.model.*;
import be.ac.vub.cocompose.eclipse.model.EclipseElement;
import be.ac.vub.cocompose.eclipse.model.IPropertyDescriptorCreator;
import be.ac.vub.cocompose.eclipse.model.datatypes.MultiplicityPropertySource;
import be.ac.vub.cocompose.eclipse.model.datatypes.PointPropertySource;
import be.ac.vub.cocompose.lang.ModelElementException;
import be.ac.vub.cocompose.lang.Properties;
import be.ac.vub.cocompose.lang.datatypes.Multiplicity;
import be.ac.vub.cocompose.lang.impl.core.DefaultRole;

/**
 * The Concept language element for Eclipse.
 * @author Dennis Wagelaar
 * @version 0.2.1, 13/01/2004
 * @since 0.2.0
 */
public class EclipseRole extends DefaultRole implements IPropertyDescriptorCreator 
{
  private EclipseElement element = new EclipseElement(this);

  /**
   * Creates the property descriptors.
   * @return the property descriptors.
   */
  public List createDescriptors()
  {
    List descs = new Vector();
    // name
    TextPropertyDescriptor desc = new TextPropertyDescriptor(Properties.ID_NAME, "name");
    desc.setValidator(new NameValidator(this));
    descs.add(desc);
    // position
    descs.add(new PropertyDescriptor(Properties.ID_POSITION, "position"));
    // background color
    descs.add(new ColorPropertyDescriptor(Properties.ID_BGCOLOR, "background color"));
    // multiplicity
    descs.add(new PropertyDescriptor(Properties.ID_MULTIPLICITY, "multiplicity"));
    // iconUri
    desc = new TextPropertyDescriptor(Properties.ID_ICON_URI, "icon URI");
    desc.setValidator(new IconValidator(this));
    descs.add(desc);
    return descs;
  }

  /**
   * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java.lang.Object)
   */
  public Object getPropertyValue(Object id)
  {
    if (Properties.ID_NAME.equals(id))
    {
      if (getName() == null)
        return new String();
      else
        return new String(getName());
    }
    if (Properties.ID_POSITION.equals(id))
      return new PointPropertySource(new Point(getPosition().x, getPosition().y));
    if (Properties.ID_BGCOLOR.equals(id))
      return new RGB(getBackgroundColor().getRed(),
                     getBackgroundColor().getGreen(),
                     getBackgroundColor().getBlue());
    if (Properties.ID_MULTIPLICITY.equals(id))
      return new MultiplicityPropertySource(getMultiplicity());
    if (Properties.ID_ICON_URI.equals(id))
    {
      if (getIconUri() == null)
        return new String();
      else
        return new String(getIconUri());
    }
    return element.getPropertyValue(id);
  }

  /**
   * Sets the value of a given property with the value supplied.
   * Also fires a property change if necessary.
   * @param id - Name of the parameter to be changed.
   * @param value - Value to be set to the given parameter.
   * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java.lang.Object, java.lang.Object)
   */
  public void setPropertyValue(Object id, Object value)
  {
    if (Properties.ID_NAME.equals(id))
    {
      try
      {
        setName((String) value);
      }
      catch (ModelElementException e)
      { // should never occur because of NameValidator
        e.printStackTrace();
      }
    }
    else if (Properties.ID_POSITION.equals(id))
    {
      PointPropertySource ptPS = (PointPropertySource) value;
      setPosition(new java.awt.Point(ptPS.getValue().x, ptPS.getValue().y));
    }
    else if (Properties.ID_BGCOLOR.equals(id))
    {
      java.awt.Color bgcolor = new java.awt.Color(((RGB) value).red,
                                                  ((RGB) value).green,
                                                  ((RGB) value).blue);
      setBackgroundColor(bgcolor);
    }
    else if (Properties.ID_MULTIPLICITY.equals(id))
    {
      MultiplicityPropertySource mPS = (MultiplicityPropertySource) value;
      setMultiplicity(mPS.getValue());
    }
    else if (Properties.ID_ICON_URI.equals(id))
    {
      if (((String) value).length() == 0)
        setIconUri(null);
      else
        setIconUri((String) value);
    }
    else element.setPropertyValue(id, value);
  }

  /**
   * @see org.eclipse.ui.views.properties.IPropertySource#resetPropertyValue(java.lang.Object)
   */
  public void resetPropertyValue(Object id)
  {
    if (Properties.ID_POSITION.equals(id))
      setPosition(new java.awt.Point(0, 0));
    else if (Properties.ID_BGCOLOR.equals(id))
      setBackgroundColor(getDefaultBackgroundColor());
    else if (Properties.ID_MULTIPLICITY.equals(id))
      setMultiplicity(new Multiplicity());
    else
      element.resetPropertyValue(id);
  }

  /////////////////////////////////////////////////////////////////////////////
  // Delegate to EclipseElement
  /////////////////////////////////////////////////////////////////////////////

  /**
   * @see org.eclipse.ui.views.properties.IPropertySource#isPropertySet(java.lang.Object)
   */
  public boolean isPropertySet(Object id)
  {
    return element.isPropertySet(id);
  }

  /**
   * @see org.eclipse.ui.views.properties.IPropertySource#getEditableValue()
   */
  public Object getEditableValue()
  {
    return element.getEditableValue();
  }

  /**
   * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
   */
  public IPropertyDescriptor[] getPropertyDescriptors()
  {
    return element.getPropertyDescriptors();
  }
}
