/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package be.ac.vub.cocompose.eclipse.model.datatypes;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.eclipse.core.internal.runtime.Assert;
import org.eclipse.ui.views.properties.ComboBoxPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource;

import be.ac.vub.cocompose.lang.core.ModelElement;
import be.ac.vub.cocompose.lang.core.Namespace;

/**
 * Eclipse property source for the Concept part-of relationship.
 * @author Dennis Wagelaar
 * @version 0.2.4, 22/01/2004
 * @since 0.2.4
 */
public class ModelElementPropertySource implements IPropertySource
{
  private Namespace namespace = null;
  private IPropertyDescriptor[] desc = null;
  private List values = null;
  private List elements = null;
  private ModelElement element = null;
  private Object id;
  private String name;
  
  /**
   * Creates a PartOfPropertySource
   * @param value the current property value.
   * @param namespace the Namespace in which to look for the property.
   * @param elementClass the Class restriction for possible values.
   * @param id the property id.
   * @param name the property name to be displayed.
   */
  public ModelElementPropertySource(ModelElement value, Namespace namespace, 
    Class elementClass, Object id, String name)
  {
    Assert.isNotNull(namespace);
    Assert.isNotNull(id);
    Assert.isNotNull(name);
    this.element = value;
    this.namespace = namespace;
    this.id = id;
    this.name = name;
    if (elementClass != null)
      this.elements = namespace.getDeepOwnedElements(elementClass);
    else
      this.elements = namespace.getDeepOwnedElements(ModelElement.class);
    this.values = createValues();
  }

  /**
   * @see org.eclipse.ui.views.properties.IPropertySource#getEditableValue()
   */
  public Object getEditableValue()
  {
    return this;
  }

  /**
   * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
   */
  public IPropertyDescriptor[] getPropertyDescriptors()
  {
    try
    {
      if (desc == null)
      {
        desc = createDescriptorsArray(createPropertyDescriptors());
      }
    }
    catch (Exception e)
    { // do nothing
      e.printStackTrace();
    }
    return desc;
  }

  /**
   * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java.lang.Object)
   */
  public Object getPropertyValue(Object id)
  {
    try
    {
      if (element != null)
        return new Integer(elements.indexOf(element) + 1);
      else
        return new Integer(0);
    }
    catch (NumberFormatException e)
    { // do nothing
    }
    return null;
  }

  /**
   * @see org.eclipse.ui.views.properties.IPropertySource#isPropertySet(java.lang.Object)
   */
  public boolean isPropertySet(Object id)
  {
    return true;
  }

  /**
   * @see org.eclipse.ui.views.properties.IPropertySource#resetPropertyValue(java.lang.Object)
   */
  public void resetPropertyValue(Object id)
  {
    element = null;
  }

  /**
   * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java.lang.Object, java.lang.Object)
   */
  public void setPropertyValue(Object id, Object value)
  {
    try
    {
      int valueIndex = ((Integer) value).intValue();
      if (valueIndex > 0)
        element = (ModelElement) elements.get(valueIndex - 1);
      else
        element = null;
    }
    catch (NumberFormatException e)
    { // do nothing
      e.printStackTrace();
    }
  }
 
  /**
   * Gets the ModelElement.
   * @return the ModelElement.
   */
  public ModelElement getModelElement()
  {
    return element;  
  }

  /**
   * @see java.lang.Object#toString()
   */
  public String toString()
  {
    String value;
    if (element != null)
      value = element.toString();
    else
      value = "<<none>>";
    return value;
  }

  /**
   * Creates the property descriptors list
   * @return the property descriptors list
   */
  private List createPropertyDescriptors()
  {
    List descs = new Vector();
    String[] values = createStringArray(this.values);
    ComboBoxPropertyDescriptor desc = new ComboBoxPropertyDescriptor(id, name, values);
    desc.setLabelProvider(new IndexedLabelProvider(elements));
    descs.add(desc);
    return descs;
  }

  /**
   * Creates the available values.
   * @return the list of available values.
   */
  private List createValues()
  {
    List values = new Vector();
    values.add("<<none>>");
    Iterator elts = this.elements.iterator();
    while (elts.hasNext())
    {
      values.add(elts.next().toString());
    }
    return values;
  }

  /**
   * Creates the property descriptors array from a Collection.
   * @param desc - the descriptors collection.
   * @return the property descriptors array.
   */
  private IPropertyDescriptor[] createDescriptorsArray(Collection desc)
  {
    IPropertyDescriptor[] idesc = new IPropertyDescriptor[desc.size()];
    int index = 0;
    Iterator descIt = desc.iterator();
    while (descIt.hasNext())
    {
      idesc[index] = (IPropertyDescriptor) descIt.next();
      index++;
    }
    return idesc;
  }

  /**
   * Creates a string array from a Collection.
   * @param coll - the collection.
   * @return the string array.
   */
  private String[] createStringArray(Collection coll)
  {
    String[] strings = new String[coll.size()];
    int index = 0;
    Iterator collIt = coll.iterator();
    while (collIt.hasNext())
    {
      strings[index] = (String) collIt.next();
      index++;
    }
    return strings;
  }
}
