/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003 - 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package be.ac.vub.cocompose.lang.core;

import java.util.List;
import be.ac.vub.cocompose.lang.ModelElementException;

/**
 * Default container and namespace mechanism interface for all CoCompose language elements.
 * @version 0.2.7, 16/04/2004
 * @since 0.2.0
 * @author Dennis Wagelaar 
 */
public interface Namespace extends be.ac.vub.cocompose.lang.core.ModelElement
{
  /**
   * Returns the elements within this namespace.
   * @return the List of owned elements.
   */
  public List getOwnedElements();

  /**
   * Returns the elements of a given class within this namespace.
   * @param elementClass the element class.
   * @return the List of owned elements.
   */
  public List getOwnedElements(Class elementClass);

  /**
   * Returns the elements of a given class within this namespace and nested namespaces.
   * @param elementClass the element class.
   * @return the List of owned elements.
   */
  public List getDeepOwnedElements(Class elementClass);

  /**
   * Adds an element to this namespace.
   * @param modelElement - the element to add.
   * @exception ModelElementException - if the name of the element is already used within this namespace. 
   */
  public void addOwnedElement(ModelElement modelElement) throws ModelElementException ;

  /**
   * Adds an element to this namespace.
   * @param index - the index at which to add the element.
   * @param modelElement - the element to add.
   * @exception ModelElementException - if the name of the element is already used within this namespace. 
   */
  public void addOwnedElement(int index, ModelElement modelElement) throws ModelElementException ;

  /**
   * Removes a element from this namespace.
   * @param modelElement - the element to remove. 
   */
  public void removeOwnedElement(ModelElement modelElement);

  /**
   * Retrieves an owned element by its name.
   * @param name - the name of the element to retrieve.
   * @return the element with the given name or null if it doesn't exist. 
   */
  public ModelElement getOwnedElement(String name);

  /**
   * Checks whether a name is already registered within this namespace.
   * @param name - the name to check.
   * @return true if and only if the name is already registered. 
   */
  public boolean isRegistered(String name);

  /**
   * Registers an element name within this namespace. This method should be called by contained ModelElements that want to change their name.
   * @param oldName - the name to change.
   * @param newName - the name to change into.
   * @exception ModelElementException - if the new name is already used or if the old name does not exist in this namespace.
   */
  public void registerNameChange(String oldName, String newName) throws ModelElementException;

  /**
   * Returns the Id of the given ModelElement or "orphan" if the element is not contained within this namespace.
   * @param modelElement - the element to get the Id for.
   * @return The element Id or null. 
   */
  public String getOwnedElementId(ModelElement modelElement);
}
