package be.ac.vub.cocompose.lang.impl.constraints;

import be.ac.vub.cocompose.lang.ModelElementException;
import be.ac.vub.cocompose.lang.impl.core.DefaultNamespace;
import be.ac.vub.cocompose.lang.constraints.Constraint;
import be.ac.vub.cocompose.lang.constraints.CompositeConstraint;
import be.ac.vub.cocompose.lang.ModelVisitor;

/**
 * Represents a general constraint.
 * @author Dennis Wagelaar 
 * @version 0.2.0
 * @since 0.2.0
 */
abstract public class DefaultConstraint extends DefaultNamespace implements Constraint 
{
  private CompositeConstraint ownerConstraint = null;
  private boolean inverted;

  /**
   * Accepts a visitor.
   * @param modelVisitor The visitor to accept.*/
  public void accept(ModelVisitor modelVisitor) throws ModelElementException 
  {
    modelVisitor.visitConstraint(this);
  }

  /**
   * Invokes superclass accept().
   * @param modelVisitor The visitor to accept.
   * @see #accept(ModelVisitor)*/
  public void acceptSuper(ModelVisitor modelVisitor) throws ModelElementException
  {
    super.accept(modelVisitor);
  }

  /**
   * Checks whether the constraint holds. This method should not be overriden. Override the concreteCheck method instead.
   * @exception ModelElementException - if the hard constraint doesn't hold.
   */
  public void check() throws ModelElementException
  {
    if (isInverted())
    {
      try
      {
        concreteCheck();
        throw new ModelElementException("Constraint " + getName() + " violated",
          this, ModelElementException.ERROR);
      }
      catch (ModelElementException e)
      {
      }
    }
    else
    {
      concreteCheck();
    }
  }

  /**
   * Gets the owner constraint.
   * @return the owner constraint.
   */
  public CompositeConstraint getOwnerConstraint()
  {
    return ownerConstraint;
  }

  /**
   * Sets the owner constraint.
   * @param ownerConstraint - the owner constraint.
   */
  public void setOwnerConstraint(CompositeConstraint ownerConstraint)
  {
    this.ownerConstraint = ownerConstraint;
  }


  /**
   * Gets the inverted attribute.
   * @return true if and only if the constraint is inverted.
   */
  public boolean isInverted()
  {
    return inverted;
  }

  /**
   * Sets the inverted attribute.
   * @param inverted - if true, the constraint is inverted.
   */
  public void setInverted(boolean inverted)
  {
    this.inverted = inverted;
  }

  /**
   * Checks whether the constraint holds. Subclasses should override this method to implement the actual check.
   * @exception ModelElementException - if the constraint doesn't hold.
   */
  protected abstract void concreteCheck() throws ModelElementException;
}
