/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package be.ac.vub.cocompose.lang.impl.core;

import be.ac.vub.cocompose.lang.ModelElementException;
import be.ac.vub.cocompose.lang.ModelVisitor;
import be.ac.vub.cocompose.lang.Properties;
import be.ac.vub.cocompose.lang.core.Model;
import be.ac.vub.cocompose.lang.core.ModelElement;
import be.ac.vub.cocompose.lang.core.Namespace;
import be.ac.vub.cocompose.lang.impl.DefaultElement;

/**
 * Root class for each CoCompose language element.
 * @author Dennis Wagelaar
 * @version 0.2.4, 22/01/2004
 * @since 0.2.0
 */
abstract public class DefaultModelElement extends DefaultElement implements ModelElement 
{
  private String name = null;
  private Namespace namespace = null;

  /**
   * Gets the name of this ModelElement.
   * @return the name of this ModelElement.
   */
  public String getName()
  {
    return name;
  }

  /**
   * Sets the name of this ModelElement.
   * @param name - the name of this ModelElement.
   * @exception ModelElementException - if the name is already used within the namespace.
   */
  public void setName(String name) throws ModelElementException
  {
    if (namespace != null)
      namespace.registerNameChange(this.name, name);
    this.name = name;
    firePropertyChange(Properties.ID_NAME, null, name);
  }

  /**
   * Gets the namespace that contains this element.
   * @return the namespace.
   */
  public Namespace getNamespace()
  {
    return namespace;
  }

  /**
   * Sets the namespace that contains this element.
   * @param namespace - the namespace.
   */
  public void setNamespace(Namespace namespace)
  {
    this.namespace = namespace;
  }

  /**
   * Gets the Id of this element.
   * @return the Id.
   */
  public String getId()
  {
    if (namespace != null)
      return namespace.getOwnedElementId(this);
    else
      return "root";
  }

  /**
   * Ignored.
   * @param id - the Id.
   */
  public void setId(String id)
  { //ignored
  }

  /**
   * Accepts a visitor.
   * @param modelVisitor The visitor to accept.
   */
  public void accept(ModelVisitor modelVisitor) throws ModelElementException 
  {
    modelVisitor.visitModelElement(this);
  }
  
  /**
   * @see java.lang.Object#toString()
   */
  public String toString()
  {
    String value = getName();
    Namespace ns = getNamespace();
    if (ns == null)
      return value;
    while (ns.getNamespace() != null)
    {
      value = ns.getName() + "." + value;
      ns = ns.getNamespace();
    }
    return value;
  }

  /**
   * Gets the root model for this element, if any.
   * @return the root model, if any.
   */
  public Model getModel()
  {
    Namespace ns = getNamespace();
    while (ns.getNamespace() != null)
    {
      ns = ns.getNamespace();
    }
    if (ns instanceof Model)
      return (Model) ns;
    else
      return null;
  }
}
