/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package be.ac.vub.cocompose.lang.impl.core;

import java.util.List;
import java.util.Vector;
import be.ac.vub.cocompose.lang.ModelElementException;
import be.ac.vub.cocompose.lang.ModelVisitor;
import be.ac.vub.cocompose.lang.Properties;
import be.ac.vub.cocompose.lang.core.RefinedElement;
import be.ac.vub.cocompose.lang.core.Refinement;
import be.ac.vub.cocompose.lang.core.ModelElement;

/**
 * Represents a language element that can have refinements.
 * @author Dennis Wagelaar 
 * @version 0.2.1, 13/01/2004
 * @since 0.2.0
 */
abstract public class DefaultRefinedElement extends DefaultRelationElement implements RefinedElement 
{
  private Vector refinements = new Vector();
  private Vector inheritsFrom = new Vector();
  private Vector superimposesOn = new Vector();
  private String iconUri = null;

  /**
   * Accepts a visitor.
   * @param modelVisitor The visitor to accept.
   */
  public void accept(ModelVisitor modelVisitor) throws ModelElementException 
  {
    modelVisitor.visitRefinedElement(this);
  }

  /**
   * Adds an element to this namespace.
   * @param index - the index at which to add the element.
   * @param modelElement - the element to add.
   * @exception ModelElementException - if the name of the element is already used within this namespace. 
   */
  public void addOwnedElement(int index, ModelElement modelElement) throws ModelElementException
  {
    super.addOwnedElement(index, modelElement);
    if (modelElement instanceof Refinement)
      addRefinement(translateIndex(index, refinements), (Refinement) modelElement);
  }

  /**
   * Removes a element from this namespace.
   * @param modelElement - the element to remove. 
   */
  public void removeOwnedElement(ModelElement modelElement)
  {
    if (modelElement instanceof Refinement)
      removeRefinement((Refinement) modelElement);
    super.removeOwnedElement(modelElement);
  }

  /**
   * Adds a refinement to this stereotype.
   * @param refinement - the refinement to add. 
   * @param index - the index at which to add the element.
   * @exception ModelElementException - if the name of the element is already used within this namespace.
   */
  private void addRefinement(int index, Refinement refinement) throws ModelElementException 
  {
    if (index > -1)
      refinements.add(index, refinement);
    else
      refinements.add(refinement);
    refinement.setRefinedElement(this);
    fireStructureChange(Properties.ID_REFINEMENTS, refinement);
  }

  /**
   * Removes a refinement from this stereotype.
   * @param refinement - the refinement to remove. 
   */
  private void removeRefinement(Refinement refinement)
  {
    refinements.remove(refinement);
    fireStructureChange(Properties.ID_REFINEMENTS, refinement);
  }

  /**
   * Returns the refinements for this stereotype.
   * @return a copy of the List of refinements.
   */
  public List getRefinements()
  {
    return (List) refinements;
  }

  /**
   * Gets the inherited elements.
   * @return the list of inherited elements.
   */
  public List getInheritsFrom()
  {
    return (List) inheritsFrom;
  }

  /**
   * Adds an inherited element to this concept.
   * @param inheritsFrom the inherited element to add.
   */
  public void addInheritsFrom(RefinedElement inheritsFrom)
  {
    addInheritsFrom(-1, inheritsFrom);
  }

  /**
   * Adds an inherited element to this concept.
   * @param index the index at which to add the element.
   * @param inheritsFrom the inherited element to add.
   */
  public void addInheritsFrom(int index, RefinedElement inheritsFrom)
  {
    if (index > -1)
      this.inheritsFrom.add(index, inheritsFrom);
    else
      this.inheritsFrom.add(inheritsFrom);
    fireStructureChange(Properties.ID_INHERITS_FROM, inheritsFrom);
  }

  /**
   * Removes an inherited element from this concept.
   * @param inheritsFrom the inherited element to remove.
   */
  public void removeInheritsFrom(RefinedElement inheritsFrom)
  {
    this.inheritsFrom.remove(inheritsFrom);
    fireStructureChange(Properties.ID_INHERITS_FROM, inheritsFrom);
  }

  /**
   * Gets the superimposition elements.
   * @return the list of superimposition elements.
   */
  public List getSuperimposesOn()
  {
    return (List) superimposesOn;
  }

  /**
   * Adds a superimposition element to this concept.
   * @param superimposesOn the superimposition element to add.
   */
  public void addSuperimposesOn(RefinedElement superimposesOn)
  {
    addSuperimposesOn(-1, superimposesOn);
  }

  /**
   * Adds a superimposition element to this concept.
   * @param index the index at which to add the element.
   * @param superimposesOn the superimposition element to add.
   */
  public void addSuperimposesOn(int index, RefinedElement superimposesOn)
  {
    if (index > -1)
      this.superimposesOn.add(index, superimposesOn);
    else
      this.superimposesOn.add(superimposesOn);
    fireStructureChange(Properties.ID_SUPERIMPOSES_ON, superimposesOn);
  }

  /**
   * Removes a superimposition element from this concept.
   * @param superimposesOn the superimposition element to remove.
   */
  public void removeSuperimposesOn(RefinedElement superimposesOn)
  {
    this.superimposesOn.remove(superimposesOn);
    fireStructureChange(Properties.ID_SUPERIMPOSES_ON, superimposesOn);
  }

  /**
   * Gets the icon image URI.
   * @return the icon URI.
   */
  public String getIconUri()
  {
    return iconUri;
  }

  /**
   * Gets the icon image URI.
   * @param uri the icon URI.
   */
  public void setIconUri(String uri)
  {
    this.iconUri = uri;
    firePropertyChange(Properties.ID_ICON_URI, null, iconUri);
  }
}
