/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package be.ac.vub.cocompose.lang.impl.core;

import java.awt.Color;
import java.util.List;
import java.util.Vector;
import be.ac.vub.cocompose.lang.ModelVisitor;
import be.ac.vub.cocompose.lang.ModelElementException;
import be.ac.vub.cocompose.lang.Properties;
import be.ac.vub.cocompose.lang.core.Role;
import be.ac.vub.cocompose.lang.core.Concept;
import be.ac.vub.cocompose.lang.datatypes.Multiplicity;

/**
 * The Solution Role language element.
 * @author Dennis Wagelaar 
 * @version 0.2.0, 07/01/2004
 * @since 0.2.0
 */
public class DefaultRole extends DefaultRefinedElement implements Role
{
  private static final Color[] DEFAULT_COLORS = {
    new Color(255, 204, 153), // orange
    new Color( 35, 225, 223), // cyan
    new Color(111, 114, 255), // purple
    new Color(111, 255, 111), // green
    new Color(154,  10,  10), // red
    new Color(250, 113,  30), // dark orange
    new Color(  0, 150, 255)  // blue
  };
  private static int FillColorIndex = 0;

  private Multiplicity multiplicity = new Multiplicity();
  private Vector parts = new Vector();

  /**
   * Creates a DefaultRole.
   */
  public DefaultRole()
  {
    this.setBackgroundColor(DEFAULT_COLORS[FillColorIndex]);
    FillColorIndex++;
    if (FillColorIndex >= DEFAULT_COLORS.length)
      FillColorIndex = 0; // modulo
  }

  /**
   * Accepts a visitor.
   * @param modelVisitor The visitor to accept.
   */
  public void accept(ModelVisitor modelVisitor) throws ModelElementException 
  {
    modelVisitor.visitRole(this);
  }

  /**
   * Adds a concept to the role part.
   * @param concept - the concept to add. 
   */
  public void addPart(Concept concept)
  {
    parts.add(concept);
  }

  /**
   * Adds a concept to the role part.
   * @param index the index at which to add the element.
   * @param concept - the concept to add. 
   */
  public void addPart(int index, Concept concept)
  {
    if (this.parts.contains(concept))
      return;
    if (index > -1)
      this.parts.add(index, concept);
    else
      this.parts.add(concept);
    fireStructureChange(Properties.ID_PARTS, concept);
  }

  /**
   * Removes a concept from the role part.
   * @param concept - the concept to remove. 
   */
  public void removePart(Concept concept)
  {
    parts.remove(concept);
    fireStructureChange(Properties.ID_PARTS, concept);
  }

  /**
   * Returns the concepts that belong to this role part.
   * @return the list of concepts.
   */
  public List getParts()
  {
    return (List) parts;
  }

  /**
   * Returns the multiplicity constraint for this solution role.
   * @return the multiplicity constraint.
   */
  public Multiplicity getMultiplicity()
  {
    return multiplicity;
  }

  /**
   * Sets the multiplicity constraint for this solution role.
   * @param multiplicity the multiplicity constraint.
   */
  public void setMultiplicity(Multiplicity multiplicity)
  {
    if (multiplicity == null) // spit out any nulls ;-)
      multiplicity = new Multiplicity();
    this.multiplicity = multiplicity;
    firePropertyChange(Properties.ID_MULTIPLICITY, null, multiplicity);
  }
}
